/* Examples and Documentation for
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#pragma once

#include "helpers.h"

namespace basic_controller_model_1 {

	////// NOTE OP-E: for every model, create unique GUIDs using the Tools -> Create GUID box
	constexpr GUID model_id = { 0xeebef929, 0x1a1c, 0x4242, { 0xb0, 0x34, 0xa9, 0x68, 0x69, 0x39, 0xa1, 0xea } };	// {EEBEF929-1A1C-4242-B034-A9686939A1EA}
	constexpr GUID bolus_id = { 0x1fcd6062, 0xa0b3, 0x41a6, { 0xa5, 0xbc, 0x2a, 0xf8, 0x6e, 0x22, 0x9a, 0xb3 } };	// {1FCD6062-A0B3-41A6-A5BC-2AF86E229AB3}
	constexpr GUID basal_id = { 0x8afed0cb, 0x455e, 0x4ed3, { 0xa4, 0x8a, 0xf3, 0xb9, 0x23, 0xb, 0xcf, 0x7f } };	// {8AFED0CB-455E-4ED3-A48A-F3B9230BCF7F}

	////// NOTE OP-E:
	// the following code defines parameters structure - these parameters will be available to your code in the implementation
	// the order of the parameters must match the order you defined in descriptor.cpp
	DEFINE_PARAMETERS_STRUCTURE(TParameters /* leave the name as-is */, 2 /* this is the count of parameters - it matches the number of fields below */, {
		double carb_ratio;
	});
}

namespace basic_controller_model_2 {

	////// NOTE OP-E: for every model, create unique GUIDs using the Tools -> Create GUID box
	constexpr GUID model_id = { 0xb7a73e87, 0xc057, 0x4ea2, { 0x84, 0xc2, 0x5, 0xaa, 0x9a, 0x5, 0x36, 0xab } };		// {B7A73E87-C057-4EA2-84C2-05AA9A0536AB}
	constexpr GUID bolus_id = { 0xc75c485, 0x2541, 0x4b31, { 0xb5, 0xba, 0xb4, 0xfa, 0x60, 0xc4, 0x92, 0xe0 } };	// {0C75C485-2541-4B31-B5BA-B4FA60C492E0}
	constexpr GUID basal_id = { 0x28dec59f, 0xf5e7, 0x4226, { 0xb8, 0x5, 0xf0, 0x73, 0x57, 0xa5, 0x6a, 0xa5 } };	// {28DEC59F-F5E7-4226-B805-F07357A56AA5}

	////// NOTE OP-E:
	// the following code defines parameters structure - these parameters will be available to your code in the implementation
	// the order of the parameters must match the order you defined in descriptor.cpp
	DEFINE_PARAMETERS_STRUCTURE(TParameters /* leave the name as-is */, 2 /* this is the count of parameters - it matches the number of fields below */, {
		double carb_ratio;
		double ig_factor;
	});
}

////// NOTE OP-E: when creating a new model, try to follow the pattern mentioned above (the namespace with unique name, GUIDs for model and signals, defined parameters structure)
