/* Examples and Documentation for
 * SmartCGMS - continuous glucose monitoring and controlling framework
 * https://diabetes.zcu.cz/
 *
 * Copyright (c) since 2018 University of West Bohemia.
 *
 * Contact:
 * diabetes@mail.kiv.zcu.cz
 * Medical Informatics, Department of Computer Science and Engineering
 * Faculty of Applied Sciences, University of West Bohemia
 * Univerzitni 8, 301 00 Pilsen
 * Czech Republic
 * 
 * 
 * Purpose of this software:
 * This software is intended to demonstrate work of the diabetes.zcu.cz research
 * group to other scientists, to complement our published papers. It is strictly
 * prohibited to use this software for diagnosis or treatment of any medical condition,
 * without obtaining all required approvals from respective regulatory bodies.
 *
 * Especially, a diabetic patient is warned that unauthorized use of this software
 * may result into severe injure, including death.
 *
 *
 * Licensing terms:
 * Unless required by applicable law or agreed to in writing, software
 * distributed under these license terms is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

////// NOTE OP-E: you don't need to change anything in this file

#include "helpers.h"

std::vector<TModel_Descriptor_Helper> Defined_Models_Storage;
std::map<GUID, TModel_Factory> Model_Factories;

std::vector<scgms::TModel_Descriptor> model_descriptors;
std::vector<scgms::TSignal_Descriptor> signal_descriptors;

void transcribe_entities() {

	for (auto& dsc : Defined_Models_Storage) {
		model_descriptors.push_back(scgms::TModel_Descriptor{
			dsc.id,
			dsc.flags,
			dsc.description.c_str(),
			nullptr,

			dsc.total_number_of_parameters,
			dsc.number_of_segment_specific_parameters,
			dsc.parameter_types.data(),
			dsc.parameter_ui_names.data(),
			nullptr,

			dsc.lower_bound.data(),
			dsc.default_values.data(),
			dsc.upper_bound.data(),

			dsc.calculated_signal_ids.size(),
			dsc.calculated_signal_ids.data(),
			dsc.reference_signal_ids.data()
		});

		for (auto& sd : dsc.signal_descriptors) {
			signal_descriptors.push_back(sd);
		}
	}

}

namespace {
	bool entities_initialized = false;
}

void lazy_init_entities() {
	if (entities_initialized) {
		return;
	}

	init_entities();
	transcribe_entities();

	entities_initialized = true;
}

/*
 * Discrete model library interface implementations
 */

extern "C" HRESULT IfaceCalling do_get_model_descriptors(scgms::TModel_Descriptor * *begin, scgms::TModel_Descriptor * *end) {

	lazy_init_entities();

	return do_get_descriptors(model_descriptors, begin, end);
}

extern "C" HRESULT IfaceCalling do_get_signal_descriptors(scgms::TSignal_Descriptor * *begin, scgms::TSignal_Descriptor * *end) {

	lazy_init_entities();

	return do_get_descriptors(signal_descriptors, begin, end);
}

extern "C" HRESULT IfaceCalling do_create_discrete_model(const GUID * model_id, scgms::IModel_Parameter_Vector * parameters, scgms::IFilter * output, scgms::IDiscrete_Model * *model) {

	if (!model_id) {
		return E_INVALIDARG;
	}

	auto itr = Model_Factories.find(*model_id);
	if (itr != Model_Factories.end()) {
		return itr->second(parameters, output, model);
	}

	return E_NOTIMPL;
}
